@@ -162,7 +162,7 @@ def publish(method):
         else:
             # another thread, run via message bus
             self._allow_read.wait()
-            response = queue.Queue(maxsize=4096)
+            response = queue.Queue()
             request = cmsg_req(response, *argv, **kwarg)
             self.ndb._event_queue.put((request, ))
             while True:
@@ -808,8 +808,18 @@ class DBSchema(object):
                 if wref() is None:
                     del self.snapshots[name]
                     if name.startswith('ifinfo_'):
-                        self.execute('DROP VIEW %s' % name[7:])
-                    self.execute('DROP TABLE %s' % name)
+                        try:
+                            self.execute('DROP VIEW %s' % name[7:])
+                        except Exception as e:
+                            self.log.debug(
+                                'failed to remove view %s: %s'
+                                % (name[7:], e))
+                    try:
+                        self.execute('DROP TABLE %s' % name)
+                    except Exception as e:
+                        self.log.debug(
+                            'failed to remove table %s: %s'
+                            % (name, e))
 
             # clean marked routes
             self.execute('DELETE FROM routes WHERE '
