@@ -1,5 +1,6 @@
 from socket import AF_INET
 from socket import AF_INET6
+from collections import OrderedDict
 from pyroute2.common import AF_MPLS
 from pyroute2.common import basestring
 from pyroute2.netlink.rtnl import rt_type
@@ -22,10 +23,10 @@ encap_types = {'mpls': 1,
                'seg6local': 7}
 
 
-class IPRequest(dict):
+class IPRequest(OrderedDict):
 
     def __init__(self, obj=None):
-        dict.__init__(self)
+        super(IPRequest, self).__init__()
         if obj is not None:
             self.update(obj)
 
@@ -70,7 +71,23 @@ class IPRuleRequest(IPRequest):
                      .get(value, (FR_ACT_NAMES
                                   .get('FR_ACT_' + value.upper(), value))))
 
-        dict.__setitem__(self, key, value)
+        super(IPRuleRequest, self).__setitem__(key, value)
+
+
+class IPAddrRequest(IPRequest):
+
+    def __setitem__(self, key, value):
+        if key in ('preferred_lft', 'valid_lft'):
+            key = key[:-4]
+        if key in ('preferred', 'valid'):
+            super(IPAddrRequest, self).__setitem__('IFA_CACHEINFO', {})
+        super(IPAddrRequest, self).__setitem__(key, value)
+
+    def sync_cacheinfo(self):
+        cacheinfo = self.get('IFA_CACHEINFO', self.get('cacheinfo', None))
+        if cacheinfo is not None:
+            for i in ('preferred', 'valid'):
+                cacheinfo['ifa_%s' % i] = self.get(i, pow(2, 32) - 1)
 
 
 class IPRouteRequest(IPRequest):
@@ -220,6 +237,8 @@ class IPRouteRequest(IPRequest):
             srh = {}
             segs = []
             hmac = None
+            prog_fd = None
+            prog_name = None
             # Parse segs
             if srh:
                 segs = header['srh']['segs']
@@ -305,6 +324,10 @@ class IPRouteRequest(IPRequest):
                     # Add to ret the hmac key
                     srh['hmac'] = hmac & 0xffffffff
                 srh['mode'] = 'encap'
+            elif action == 'End.BPF':
+                prog_fd = header['bpf']['fd']
+                prog_name = header['bpf']['name']
+
             # Construct the new object
             ret = []
             ret.append(['SEG6_LOCAL_ACTION', {'value': action}])
@@ -326,6 +349,12 @@ class IPRouteRequest(IPRequest):
             if srh:
                 # Add the srh to ret
                 ret.append(['SEG6_LOCAL_SRH', srh])
+            if prog_fd and prog_name:
+                # Add the prog_fd and prog_name to ret
+                ret.append(['SEG6_LOCAL_BPF', {'attrs': [
+                    ['LWT_BPF_PROG_FD', prog_fd],
+                    ['LWT_BPF_PROG_NAME', prog_name],
+                ]}])
             # Done return the object
             return {'attrs': ret}
 
@@ -361,18 +390,18 @@ class IPRouteRequest(IPRequest):
             return
         # work on the rest
         if key == 'family' and value == AF_MPLS:
-            dict.__setitem__(self, 'family', value)
-            dict.__setitem__(self, 'dst_len', 20)
-            dict.__setitem__(self, 'table', 254)
-            dict.__setitem__(self, 'type', 1)
+            super(IPRouteRequest, self).__setitem__('family', value)
+            super(IPRouteRequest, self).__setitem__('dst_len', 20)
+            super(IPRouteRequest, self).__setitem__('table', 254)
+            super(IPRouteRequest, self).__setitem__('type', 1)
         elif key == 'flags' and self.get('family', None) == AF_MPLS:
             return
         elif key in ('dst', 'src'):
             if isinstance(value, (tuple, list, dict)):
-                dict.__setitem__(self, key, value)
+                super(IPRouteRequest, self).__setitem__(key, value)
             elif isinstance(value, int):
-                dict.__setitem__(self, key, {'label': value,
-                                             'bos': 1})
+                super(IPRouteRequest, self).__setitem__(key, {'label': value,
+                                                              'bos': 1})
             elif value != 'default':
                 value = value.split('/')
                 mask = None
@@ -390,15 +419,17 @@ class IPRouteRequest(IPRequest):
                     mask = int(value[1])
                 else:
                     raise ValueError('wrong address spec')
-                dict.__setitem__(self, key, dst)
+                super(IPRouteRequest, self).__setitem__(key, dst)
                 if mask is not None:
-                    dict.__setitem__(self, '%s_len' % key, mask)
+                    (super(IPRouteRequest, self)
+                     .__setitem__('%s_len' % key, mask))
         elif key == 'newdst':
-            dict.__setitem__(self, 'newdst', self.mpls_rta(value))
+            (super(IPRouteRequest, self)
+             .__setitem__('newdst', self.mpls_rta(value)))
         elif key in self.resolve.keys():
             if isinstance(value, basestring):
                 value = self.resolve[key][value]
-            dict.__setitem__(self, key, value)
+            super(IPRouteRequest, self).__setitem__(key, value)
         elif key == 'encap':
             if isinstance(value, dict):
                 # human-friendly form:
@@ -408,11 +439,12 @@ class IPRouteRequest(IPRequest):
                 #
                 # 'type' is mandatory
                 if 'type' in value and 'labels' in value:
-                    dict.__setitem__(self, 'encap_type',
-                                     encap_types.get(value['type'],
-                                                     value['type']))
-                    dict.__setitem__(self, 'encap',
-                                     self.encap_header(value))
+                    (super(IPRouteRequest, self)
+                     .__setitem__('encap_type',
+                                  encap_types.get(value['type'],
+                                                  value['type'])))
+                    (super(IPRouteRequest, self)
+                     .__setitem__('encap', self.encap_header(value)))
                 # human-friendly form:
                 #
                 # 'encap': {'type': 'seg6',
@@ -435,18 +467,20 @@ class IPRouteRequest(IPRequest):
                 #
                 # 'type', 'mode' and 'segs' are mandatory
                 if 'type' in value and 'mode' in value and 'segs' in value:
-                    dict.__setitem__(self, 'encap_type',
-                                     encap_types.get(value['type'],
-                                                     value['type']))
-                    dict.__setitem__(self, 'encap',
-                                     self.encap_header(value))
+                    (super(IPRouteRequest, self)
+                     .__setitem__('encap_type',
+                                  encap_types.get(value['type'],
+                                                  value['type'])))
+                    (super(IPRouteRequest, self)
+                     .__setitem__('encap', self.encap_header(value)))
                 elif 'type' in value and ('in' in value or 'out' in value or
                                           'xmit' in value):
-                    dict.__setitem__(self, 'encap_type',
-                                     encap_types.get(value['type'],
-                                                     value['type']))
-                    dict.__setitem__(self, 'encap',
-                                     self.encap_header(value))
+                    (super(IPRouteRequest, self)
+                     .__setitem__('encap_type',
+                                  encap_types.get(value['type'],
+                                                  value['type'])))
+                    (super(IPRouteRequest, self)
+                     .__setitem__('encap', self.encap_header(value)))
                 # human-friendly form:
                 #
                 # 'encap': {'type': 'seg6local',
@@ -467,21 +501,22 @@ class IPRouteRequest(IPRequest):
                 #
                 # 'type' and 'action' are mandatory
                 elif 'type' in value and 'action' in value:
-                    dict.__setitem__(self, 'encap_type',
-                                     encap_types.get(value['type'],
-                                                     value['type']))
-                    dict.__setitem__(self, 'encap',
-                                     self.encap_header(value))
+                    (super(IPRouteRequest, self)
+                     .__setitem__('encap_type',
+                                  encap_types.get(value['type'],
+                                                  value['type'])))
+                    (super(IPRouteRequest, self)
+                     .__setitem__('encap', self.encap_header(value)))
                 # assume it is a ready-to-use NLA
                 elif 'attrs' in value:
-                    dict.__setitem__(self, 'encap', value)
+                    super(IPRouteRequest, self).__setitem__('encap', value)
         elif key == 'via':
             # ignore empty RTA_VIA
             if isinstance(value, dict) and \
                     set(value.keys()) == set(('addr', 'family')) and \
                     value['family'] in (AF_INET, AF_INET6) and \
                     isinstance(value['addr'], basestring):
-                dict.__setitem__(self, 'via', value)
+                super(IPRouteRequest, self).__setitem__('via', value)
         elif key == 'metrics':
             if 'attrs' in value:
                 ret = value
@@ -491,7 +526,7 @@ class IPRouteRequest(IPRequest):
                     rtax = rtmsg.metrics.name2nla(name)
                     ret['attrs'].append([rtax, value[name]])
             if ret['attrs']:
-                dict.__setitem__(self, 'metrics', ret)
+                super(IPRouteRequest, self).__setitem__('metrics', ret)
         elif key == 'multipath':
             ret = []
             for v in value:
@@ -522,18 +557,18 @@ class IPRouteRequest(IPRequest):
                         nh['attrs'].append([rta, v[name]])
                 ret.append(nh)
             if ret:
-                dict.__setitem__(self, 'multipath', ret)
+                super(IPRouteRequest, self).__setitem__('multipath', ret)
         elif key == 'family':
             for d in self._mask:
                 if d not in self:
                     if value == AF_INET:
-                        dict.__setitem__(self, d, 32)
+                        super(IPRouteRequest, self).__setitem__(d, 32)
                     elif value == AF_INET6:
-                        dict.__setitem__(self, d, 128)
+                        super(IPRouteRequest, self).__setitem__(d, 128)
             self._mask = []
-            dict.__setitem__(self, key, value)
+            super(IPRouteRequest, self).__setitem__(key, value)
         else:
-            dict.__setitem__(self, key, value)
+            super(IPRouteRequest, self).__setitem__(key, value)
 
 
 class CBRequest(IPRequest):
@@ -553,7 +588,7 @@ class CBRequest(IPRequest):
             self['commands']['attrs'].\
                 append([self.msg.name2nla(key), value])
         else:
-            dict.__setitem__(self, key, value)
+            super(CBRequest, self).__setitem__(key, value)
 
 
 class IPBridgeRequest(IPRequest):
@@ -561,11 +596,12 @@ class IPBridgeRequest(IPRequest):
     def __setitem__(self, key, value):
         if key in ('vlan_info', 'mode', 'vlan_flags'):
             if 'IFLA_AF_SPEC' not in self:
-                dict.__setitem__(self, 'IFLA_AF_SPEC', {'attrs': []})
+                (super(IPBridgeRequest, self)
+                 .__setitem__('IFLA_AF_SPEC', {'attrs': []}))
             nla = ifinfmsg.af_spec_bridge.name2nla(key)
             self['IFLA_AF_SPEC']['attrs'].append([nla, value])
         else:
-            dict.__setitem__(self, key, value)
+            super(IPBridgeRequest, self).__setitem__(key, value)
 
 
 class IPBrPortRequest(dict):
@@ -629,7 +665,7 @@ class IPLinkRequest(IPRequest):
         # create IFLA_LINKINFO
         linkinfo = {'attrs': []}
         self.linkinfo = linkinfo['attrs']
-        dict.__setitem__(self, 'IFLA_LINKINFO', linkinfo)
+        super(IPLinkRequest, self).__setitem__('IFLA_LINKINFO', linkinfo)
         self.linkinfo.append(['IFLA_INFO_KIND', self.kind])
         # load specific NLA names
         cls = ifinfmsg.ifinfo.data_map.get(self.kind, None)
@@ -642,7 +678,7 @@ class IPLinkRequest(IPRequest):
         # flush deferred NLAs
         for (key, value) in self.deferred:
             if not self.set_specific(key, value):
-                dict.__setitem__(self, key, value)
+                super(IPLinkRequest, self).__setitem__(key, value)
 
         self.deferred = []
 
@@ -679,7 +715,8 @@ class IPLinkRequest(IPRequest):
                               {'vf': vfid,
                                attr: vf[attr]}))
             vflist.append(('IFLA_VF_INFO', {'attrs': vfcfg}))
-        dict.__setitem__(self, 'IFLA_VFINFO_LIST', {'attrs': vflist})
+        (super(IPLinkRequest, self)
+         .__setitem__('IFLA_VFINFO_LIST', {'attrs': vflist}))
 
     def set_specific(self, key, value):
         # FIXME: vlan hack
@@ -753,9 +790,9 @@ class IPLinkRequest(IPRequest):
             self.set_vf(value)
         elif self.kind is None:
             if key in self.common:
-                dict.__setitem__(self, key, value)
+                super(IPLinkRequest, self).__setitem__(key, value)
             else:
                 self.deferred.append((key, value))
         else:
             if not self.set_specific(key, value):
-                dict.__setitem__(self, key, value)
+                super(IPLinkRequest, self).__setitem__(key, value)
